// WebSocket Authentication Fuzzer using Bun and fuzzing libraries
import { WebSocket } from "ws"; // Bun has built-in WebSocket support
import { randomBytes } from "crypto";

// First, install the required libraries:
// bun add fuzzer @faker-js/faker fast-check

// Import fuzzing libraries
import fuzzer from "fuzzer";
import { faker } from "@faker-js/faker";
import * as fc from "fast-check";

// Configuration
const WS_URL = "wss://ios.cs.hs-rm.de:2001/health"; // Replace with your actual WebSocket URL
const MAX_REQUESTS = 200; // Number of fuzzing attempts to make
const DELAY_BETWEEN_REQUESTS = 100; // ms between requests

// Track results
const results: {
  success: number;
  failure: number;
  errors: number;
  responses: Array<{
    testId: number;
    username: string;
    password: string;
    deviceType: string;
    response: any;
    status: string;
  }>;
} = {
  success: 0,
  failure: 0,
  errors: 0,
  responses: [],
};

// Generate various username values using different strategies
function generateUsername() {
  const strategies = [
    // Standard values
    () => faker.internet.username(),
    () => faker.internet.email(),
    // Fuzzing specific values
    () => fuzzer.mutate.string("admin"),
    () => fuzzer.mutate.string("user"),
    // SQL injection attempts
    () => `' OR '1'='1`,
    () => `admin' --`,
    // Special characters
    () => Buffer.from(randomBytes(10)).toString("base64"),
    // XSS attempts
    () => `<script>alert(1)</script>`,
    // Empty and boundary values
    () => "",
    () => " ",
    () => "a".repeat(500),
    // Generated from fast-check
    () => fc.sample(fc.string(), 1)[0],
  ];

  return strategies[Math.floor(Math.random() * strategies.length)]();
}

// Generate password values
function generatePassword() {
  const strategies = [
    // Standard values
    () => faker.internet.password(),
    // Common passwords
    () => fuzzer.mutate.string("password123"),
    () => fuzzer.mutate.string("admin"),
    // SQL injection
    () => `' OR '1'='1`,
    // Special characters
    () => Buffer.from(randomBytes(10)).toString("base64"),
    // Empty and boundary values
    () => "",
    () => " ",
    () => "a".repeat(500),
    // Generated from fast-check
    () => fc.sample(fc.string(), 1)[0],
  ];

  return strategies[Math.floor(Math.random() * strategies.length)]();
}

// Generate device type values
function generateDeviceType() {
  const strategies = [
    // Standard values
    () => faker.helpers.arrayElement(["web", "watch"]),
    // Edge cases
    () => fuzzer.mutate.string("web"),
    () => fuzzer.mutate.string("watch"),
    () => "",
    () => " ",
    // XSS attempts
    () => `<script>alert('device')</script>`,
    // Special values
    () => "undefined",
    () => "null",
    // Generated from fast-check
    () => fc.sample(fc.string(), 1)[0],
  ];

  return strategies[Math.floor(Math.random() * strategies.length)]();
}

// Fuzzer function
async function runFuzzer() {
  console.log(
    `Starting WebSocket authentication fuzzer with libraries, targeting: ${WS_URL}`
  );
  console.log(`Will perform ${MAX_REQUESTS} fuzzing attempts...`);

  for (let i = 0; i < MAX_REQUESTS; i++) {
    // Generate values for each field using fuzzing libraries
    const username = generateUsername();
    const password = generatePassword();
    const deviceType = generateDeviceType();

    try {
      await testAuthentication(i, username, password, deviceType);
      // Add delay between requests
      await new Promise((resolve) =>
        setTimeout(resolve, DELAY_BETWEEN_REQUESTS)
      );
    } catch (error) {
      console.error(`Error in test #${i}: ${error}`);
      results.errors++;
    }
  }

  // Print summary
  console.log("\n--- FUZZING SUMMARY ---");
  console.log(`Total attempts: ${MAX_REQUESTS}`);
  console.log(`Successes: ${results.success}`);
  console.log(`Failures: ${results.failure}`);
  console.log(`Errors: ${results.errors}`);

  // Check for potential vulnerabilities
  const suspiciousResponses = results.responses.filter(
    (r) =>
      r.status === "success" &&
      ((typeof r.username === "string" &&
        (r.username.includes("'") || r.username.includes("<script>"))) ||
        (typeof r.password === "string" &&
          (r.password.includes("'") || r.password.includes("<script>"))))
  );

  if (suspiciousResponses.length > 0) {
    console.log("\n!!! POTENTIAL VULNERABILITIES DETECTED !!!");
    suspiciousResponses.forEach((r) => {
      console.log(`Test #${r.testId}: ${JSON.stringify(r)}`);
    });
  }
}

// Test a single authentication attempt
async function testAuthentication(
  testId: number,
  username: string,
  password: string,
  deviceType: string
) {
  return new Promise((resolve, reject) => {
    // Set connection timeout
    const timeout = setTimeout(() => {
      if (ws.readyState === WebSocket.OPEN) {
        console.log("closing!");
        ws.close();
      }
      reject(new Error("Connection timeout"));
    }, 5000);

    // Create WebSocket with cookies
    const ws = new WebSocket(WS_URL, {
      headers: {
        Cookie: `user_id=${encodeURIComponent(
          username
        )}; Authorization=${encodeURIComponent(
          password
        )}; client=${encodeURIComponent(deviceType)}`,
      },
    });

    console.log(
      `Test #${testId}: username="${username}" password="${password}" deviceType="${deviceType}"`
    );

    ws.onopen = () => {
      console.log(`Test #${testId}: Connection established`);
    };

    ws.onmessage = (event) => {
      clearTimeout(timeout);

      try {
        const response = JSON.parse(event.data.toString());
        const result = {
          testId,
          username,
          password: password,
          deviceType,
          response,
          status: response.success ? "success" : "failure",
        };

        if (response.success) {
          console.log(`Test #${testId}: SUCCESS - ${JSON.stringify(response)}`);
          results.success++;
        } else {
          console.log(`Test #${testId}: FAILURE - ${JSON.stringify(response)}`);
          results.failure++;
        }

        results.responses.push(result);
        ws.close();
        resolve(true);
      } catch (error) {
        console.log(
          `Test #${testId}: Received non-JSON response: ${event.data}`
        );
        results.responses.push({
          testId,
          username,
          password: password,
          deviceType,
          response: event.data.toString(),
          status: "unknown",
        });
        ws.close();
        resolve(null);
      }
    };

    ws.onerror = (error) => {
      clearTimeout(timeout);
      console.log(`Test #${testId}: WebSocket error`);
      results.failure++;
      ws.close();
      resolve(null); // Don't reject to continue the test
    };

    ws.onclose = () => {
      clearTimeout(timeout);
      resolve(null);
    };
  });
}

// Run the fuzzer
runFuzzer()
  .then(() => {
    console.log("Fuzzing completed.");
    process.exit(0);
  })
  .catch((error) => {
    console.error("Fuzzer crashed:", error);
    process.exit(1);
  });
